@@ -1,5 +1,5 @@
 """
-Copyright (c) 2022, Oracle and/or its affiliates.
+Copyright (c) 2023, Oracle and/or its affiliates.
 Copyright (c) 2020, Vitor Avancini
 
   Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,23 +18,34 @@ from typing import (
     Optional, List, Set
 )
 from itertools import chain
+from typing import (
+    Any,
+    Callable,
+    Dict)
+
+import agate
 
 import dbt.exceptions
 from dbt.adapters.base.relation import BaseRelation, InformationSchema
-from dbt.adapters.base.impl import GET_CATALOG_MACRO_NAME
+from dbt.adapters.base.impl import GET_CATALOG_MACRO_NAME, ConstraintSupport
 from dbt.adapters.sql import SQLAdapter
 from dbt.adapters.base.meta import available
 from dbt.adapters.oracle import OracleAdapterConnectionManager
 from dbt.adapters.oracle.column import OracleColumn
 from dbt.adapters.oracle.relation import OracleRelation
 from dbt.contracts.graph.manifest import Manifest
+from dbt.contracts.graph.nodes import ConstraintType
+from dbt.events import AdapterLogger
 
-from dbt.exceptions import raise_compiler_error
 from dbt.utils import filter_null_values
 
+from dbt.adapters.oracle.keyword_catalog import KEYWORDS
 
-import agate
+logger = AdapterLogger("oracle")
 
+# Added 6 random hex letters (56c36b) to table_a and table_b to avoid ORA-32031.
+# Some dbt test cases use relation names table_a and table_b
+# Oracle error: ORA-32031: illegal reference of a query name in WITH clause
 COLUMNS_EQUAL_SQL = '''
 with diff_count as (
     SELECT
@@ -46,15 +57,15 @@ with diff_count as (
             (SELECT {columns} FROM {relation_b} {except_op}
              SELECT {columns} FROM {relation_a})
         ) a
-), table_a as (
+), table_a_56c36b as (
     SELECT COUNT(*) as num_rows FROM {relation_a}
-), table_b as (
+), table_b_56c36b as (
     SELECT COUNT(*) as num_rows FROM {relation_b}
 ), row_count_diff as (
     select
         1 as id,
-        table_a.num_rows - table_b.num_rows as difference
-    from table_a, table_b
+        table_a_56c36b.num_rows - table_b_56c36b.num_rows as difference
+    from table_a_56c36b, table_b_56c36b
 )
 select
     row_count_diff.difference as row_count_difference,
@@ -72,6 +83,14 @@ class OracleAdapter(SQLAdapter):
     Relation = OracleRelation
     Column = OracleColumn
 
+    CONSTRAINT_SUPPORT = {
+        ConstraintType.check: ConstraintSupport.ENFORCED,
+        ConstraintType.not_null: ConstraintSupport.ENFORCED,
+        ConstraintType.unique: ConstraintSupport.ENFORCED,
+        ConstraintType.primary_key: ConstraintSupport.ENFORCED,
+        ConstraintType.foreign_key: ConstraintSupport.ENFORCED,
+    }
+
     def debug_query(self) -> None:
         self.execute("select 1 as id from dual")
 
@@ -114,7 +133,7 @@ class OracleAdapter(SQLAdapter):
             database = database.strip('"')
         expected = self.config.credentials.database
         if expected and database.lower() != expected.lower():
-            raise dbt.exceptions.NotImplementedException(
+            raise dbt.exceptions.DbtRuntimeError(
                 'Cross-db references not allowed in {} ({} vs {})'
                 .format(self.type(), database, expected)
             )
@@ -232,6 +251,62 @@ class OracleAdapter(SQLAdapter):
             ))
         return relations
 
+    @staticmethod
+    def is_valid_identifier(identifier) -> bool:
+        """Returns True if an identifier is valid
+
+        An identifier is considered valid if the following conditions are True
+
+            1. First character is alphabetic
+            2. Rest of the characters is either alphanumeric or any one of the literals '#', '$', '_'
+
+        """
+        # The first character should be alphabetic
+        if not identifier[0].isalpha():
+            return False
+        # Rest of the characters is either alphanumeric or any one of the literals '#', '$', '_'
+        idx = 1
+        while idx < len(identifier):
+            identifier_chr = identifier[idx]
+            if not identifier_chr.isalnum() and identifier_chr not in ('#', '$', '_'):
+                return False
+            idx += 1
+        return True
+
+    @available
+    def should_identifier_be_quoted(self,
+                                    identifier,
+                                    models_column_dict=None) -> bool:
+        """Returns True if identifier should be quoted else False
+
+        An identifier should be quoted in the following 3 cases:
+
+            - 1. Identifier is an Oracle keyword
+
+            - 2. Identifier is not valid according to the following rules
+                - First character is alphabetic
+                - Rest of the characters is either alphanumeric or any one of the literals '#', '$', '_'
+
+            - 3. User has enabled quoting for the column in the model configuration
+
+        """
+        if identifier.upper() in KEYWORDS:
+            return True
+        elif not self.is_valid_identifier(identifier):
+            return True
+        elif models_column_dict and identifier in models_column_dict:
+            return models_column_dict[identifier].get('quote', False)
+        elif models_column_dict and self.quote(identifier) in models_column_dict:
+            return models_column_dict[self.quote(identifier)].get('quote', False)
+        return False
+
+    @available
+    def check_and_quote_identifier(self, identifier, models_column_dict=None) -> str:
+        if self.should_identifier_be_quoted(identifier, models_column_dict):
+            return self.quote(identifier)
+        else:
+            return identifier
+
     @available
     def quote_seed_column(
             self, column: str, quote_config: Optional[bool]
@@ -239,15 +314,34 @@ class OracleAdapter(SQLAdapter):
         quote_columns: bool = False
         if isinstance(quote_config, bool):
             quote_columns = quote_config
+        elif self.should_identifier_be_quoted(column):
+            quote_columns = True
         elif quote_config is None:
             pass
         else:
-            raise_compiler_error(
-                f'The seed configuration value of "quote_columns" has an '
-                f'invalid type {type(quote_config)}'
-            )
+            raise dbt.exceptions.CompilationError(f'The seed configuration value of "quote_columns" '
+                                                  f'has an invalid type {type(quote_config)}')
 
         if quote_columns:
             return self.quote(column)
         else:
             return column
+
+    def valid_incremental_strategies(self):
+        return ["append", "merge"]
+
+    @available
+    @classmethod
+    def render_raw_columns_constraints(cls, raw_columns: Dict[str, Dict[str, Any]]) -> List:
+        rendered_column_constraints = []
+
+        for v in raw_columns.values():
+            rendered_column_constraint = [f"{v['name']}"]
+            for con in v.get("constraints", None):
+                constraint = cls._parse_column_constraint(con)
+                c = cls.process_parsed_constraint(constraint, cls.render_column_constraint)
+                if c is not None:
+                    rendered_column_constraint.append(c)
+            rendered_column_constraints.append(" ".join(rendered_column_constraint))
+
+        return rendered_column_constraints
